import scrapy, re
from datetime import datetime, date, timedelta
from scrapy.spidermiddlewares.httperror import HttpError

"""
Esta version descarga ingresando una fecha.
USO:

scrapy crawl noticias -t json --nolog -o noticias.json -a year=2017 -a month=09 -a day=13

No es recomendable para fechas de mas de un mes de antiguas.
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
	return TAG_RE.sub('', text)


class NoticiasItem(scrapy.Item):
	title = scrapy.Field()
	text = scrapy.Field()
	date = scrapy.Field()
	location = scrapy.Field()
	author = scrapy.Field()
	topic = scrapy.Field()
	url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
	name = "noticias"

	def start_requests(self):
		section_list = ['merida', 'yucatan', 'mexico', 'internacional', 'deportes',
										'espectaculos', 'imagen', 'economia', 'tecnologia', 'salud']

		year = getattr(self, 'year', None)
		month = getattr(self, 'month', None)
		day = getattr(self, 'day', None)
		self.baseURL='http://yucatan.com.mx/seccion/'
		self.date = date(int(year), int(month), int(day))
		self.parsing_month = { 'enero': 1, 'febrero': 2, 'marzo': 3, 'abril': 4, 'mayo': 5, 'junio': 6, 'julio': 7,
													 'agosto': 8, 'septiembre': 9, 'octubre': 10, 'noviembre': 11, 'diciembre': 12 }
		
		self.pages = 100
		for s in section_list:
			yield scrapy.Request(url=self.baseURL+s, callback=self.parse)



	def parse(self, response):
		if ( len(response.xpath('//a[@class="show-more-link"]/@href').extract()) > 0 ):
			for link in response.xpath('//a[@class="show-more-link"]/@href').extract():
				yield scrapy.Request(url=link, callback=self.parse_pagination)
			
		elif ( len(response.xpath('//a[@class="show-more-link"]/@href').extract()) == 0 ):
			yield scrapy.Request(url=response.url, callback=self.parse_pagination, dont_filter=True)


	def parse_pagination(self, response):
		pagination = response.xpath('//*[@class="pagination"]/a/@href').extract()
		if ( len(pagination) > 0 ):
			p = 1

			while p <= self.pages:
				if ( p == 1 ):
					yield scrapy.Request(url=response.url, callback=self.parse_link, dont_filter=True)
				elif ( p > 1 ):
					yield scrapy.Request(url=response.url+'/page/'+str(p+1), callback=self.parse_link)

				p += 1

		else:
			yield scrapy.Request(url=response.url, callback=self.parse_link, dont_filter=True)


	def parse_link(self, response):
		for entry in response.xpath('//*[@class="bp-entry"]'):
			entry_date = entry.xpath('./*[@class="bp-head"]/div/span/text()').extract_first()
			entry_date = entry_date[entry_date.rfind(',')+2:][:entry_date[entry_date.rfind(',')+2:].rfind('-')-2]

			news_date = date(int(entry_date[-4:]), self.parsing_month[entry_date[:-8][entry_date[:-8].rfind(' ')+1:]], int(entry_date[:entry_date.find(' ')]))
			link = entry.xpath('./*[@class="bp-head"]/h2/a/@href').extract_first()
			
			if news_date == self.date and link is not None:
				yield scrapy.Request(url=link, callback=self.parse_item)


	def parse_item(self, response):
		text = ''
		item = NoticiasItem()
		item['title'] = response.css('h1.entry-title::text').extract_first()
		
		d = response.css('div.base-box').css('span.entry-date::attr(datetime)').extract_first()
		if d is None:
			d = response.xpath('//meta[@itemprop="datePublished"]/@content').extract_first()
			if d is None:
				d = response.xpath('//time[@class="updated"]/@datetime').extract_first()
		
		## '-06:00' corresponde al UTC-6, zona horaria de yucatan (centro de mexico)
		if d[-6:] != '-06:00':
			d = d[:-6] + '-06:00'
		item['date'] = d

		for paragraph in response.css('div.entry-content').css('p').extract():
			text += remove_tags(paragraph) + '\n'		
		item['text'] = text
		item['topic'] = response.xpath('//*[@class="breadcrumbs-plus"]/span/a/span/text()').extract()[1]
		item['url'] = response.url
		# print item['title']
		yield item

