import scrapy, re


## scrapy crawl noticias -t json --nolog -o noticias.json -a year=2017 -a month=3 -a day=22


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
	return TAG_RE.sub('', text)


class NoticiasItem(scrapy.Item):
	title = scrapy.Field()
	text = scrapy.Field()
	date = scrapy.Field()
	location = scrapy.Field()
	author = scrapy.Field()
	topic = scrapy.Field()
	url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
	name = "noticias"
	
	def start_requests(self):
		year = getattr(self, 'year', None)
		month = getattr(self, 'month', None)
		day = getattr(self, 'day', None)
		self.baseURL='http://www.lja.mx/'+year+'/'+month.zfill(2)+'/'+day.zfill(2)
		
		yield scrapy.Request(url=self.baseURL, callback=self.parse)


	def parse(self, response):
		pagination = response.css('div.pagination').css('a::attr(href)').extract()
		
		if ( len(pagination) > 0 ):
			pagination = pagination[-1].strip('/')
			pages = int(pagination[pagination.rfind('/')+1:])
			
			for page in range(0, pages):
				if ( page == 0 ):
					yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)
				
				else:
					yield scrapy.Request(url=self.baseURL+"/page/"+str(page+1), callback=self.parse_page)
		
		else:
			yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)


	def parse_page(self, response):
		for link in response.xpath('//li[@class="infinite-post"]/a/@href').extract():
			yield scrapy.Request(url=link, callback=self.parse_item)


	def parse_item(self, response):
		item = NoticiasItem()
		text = ''
		## la fecha de la noticia ya incluye la zona horaria
		item['date'] = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
		item['title'] = response.css('h1.story-title::text').extract_first()
		item['topic'] = response.css('h3.story-cat::text').extract_first()
		item['author'] = response.xpath('//div[@id="post-info"]/span/a/text()').extract_first()
		
		for paragraph in response.xpath('//div[@id="content-area"]/p').extract():
			text += remove_tags(paragraph) + '\n'
		item['text'] = text
		item['url'] = response.url
		
		# print item['title']
		yield item

