import scrapy, re

"""
Esta version se encarga de la descarga de la pagina de La Jornada Guerrero
con url: 'http://www.lajornadaguerrero.com.mx/'

Uso:

scrapy crawl noticias -t json --nolog -o noticias.json -a year=2017 -a month=3 -a day=22
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
	return TAG_RE.sub('', text)


class NoticiasItem(scrapy.Item):
	title = scrapy.Field()
	text = scrapy.Field()
	date = scrapy.Field()
	location = scrapy.Field()
	author = scrapy.Field()
	topic = scrapy.Field()
	url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
	name = "noticias"
	
	def start_requests(self):
		self.link_list = []
		self.year = getattr(self, 'year', None)
		self.month = getattr(self, 'month', None)
		self.day = getattr(self, 'day', None)
		self.baseURL='http://www.lajornadaguerrero.com.mx/'+self.year+'/'+self.month.zfill(2)+'/'+self.day.zfill(2)
		urls = [
			self.baseURL,
								]
		for url in urls:
			yield scrapy.Request(url=url, callback=self.parse)


	def parse(self, response):
		paths = ['//div[@class="viewlet"]/h1/a/@href', '//div[@class="viewlet"]/h2/a/@href',
				 '//div[@class="viewlet"]/h3/a/@href', '//div[@class="viewlet image"]/h1/a/@href',
				 '//div[@class="viewlet image"]/h2/a/@href', '//div[@class="viewlet image"]/h3/a/@href',
				 '//div[@class="text_block_200"]/p/a/@href']
		for path in paths:
			links = response.xpath(path).extract()
			if ( len(links) > 0 ):
				for link in links:
					if not ( link in self.link_list ):
						self.link_list.append(link)
						yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_item)

		for link in response.xpath('//*[@class="text_block_200"]/h2/a/@href').extract():
			yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_page)


	def parse_page(self, response):
		paths = ['//*[@id="article_list"]/h2/a/@href',
				 '//*[@id="article_list"]/h3/a/@href']
		for path in paths:
			for link in response.xpath(path).extract():
				if not ( link in self.link_list ):
					yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_item)


	def parse_item(self, response):
		item = NoticiasItem()
		text = ''
		item['date'] = self.year+'-'+self.month.zfill(2)+'-'+self.day.zfill(2)
		item['title'] = response.xpath('//*[@class="documentContent"]/h1/text()').extract_first()
		item['topic'] = response.xpath('//*[@id="portal-breadcrumbs"]/a[2]/text()').extract()
		for paragraph in response.xpath('//*[@class="documentContent"]/p/text()').extract():
			text += paragraph
		item['text'] = text
		item['url'] = response.url
		# print item['title']
		yield item

